SystemOrganization addCategory: #'Tweak-Skins'!

CImageButton subclass: #CSkinButton
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Skins'!

!CSkinButton commentStamp: 'bf 4/30/2004 14:54' prior: 0!
A SkinButton is used as component of a Skin.

My graphic is positioned to stay fixed in my container - if you move me, I'm like a looking glass.

In my normal state, my image is not drawn at all, so the normalImage of my container is visible. Only in my other states, the corresponding image is drawn, hiding the normalImage.

An exception to this is when I'm colorized, then even the normalImage is shown. It should have some transparent parts for my color to show through.

Also, I may have a shapeForm, which is a 1-bit form where non-zero pixels indicate my shape.!

CSkinButton defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="colorize" toSet="colorize:" changeEvent="colorizeChanged">colorize</field>
	<field toGet="shapeForm" toSet="shapeForm:" changeEvent="shapeFormChanged">shapeForm</field>
</fields>'!

!CSkinButton methodsFor: 'accessing' stamp: 'bf 4/26/2004 17:39'!
colorize
	"Answer the colorize of the receiver"
	<bewareOf: #colorizeChanged>
	^self propertyValueAt: #colorize ifAbsent: [false].! !

!CSkinButton methodsFor: 'accessing' stamp: 'bf 4/15/2004 13:47'!
colorize: aValue
	"Modify the receiver's colorize"
	^self propertyValueAt: #colorize put: aValue with: #colorizeChanged! !

!CSkinButton methodsFor: 'testing' stamp: 'bf 4/30/2004 14:55'!
containsPoint: aPoint

	shapeForm ifNil: [^super containsPoint: aPoint].

	^ (shapeForm pixelValueAt: aPoint) ~= 0! !

!CSkinButton methodsFor: 'images' stamp: 'ar 4/1/2007 10:39'!
disabledImage

	^nil! !

!CSkinButton methodsFor: 'drawing' stamp: 'bf 9/7/2004 15:09'!
drawOn: aCanvas in: drawingBounds

	shapeForm ifNotNil: [^self drawShapedOn: aCanvas in: drawingBounds].

	"fill with bg pattern"
	(colorize and: [fillAlpha < 1.0]) ifTrue: [
		aCanvas drawImage: (CSkin transparentBackgroundPattern copy: drawingBounds)
			at: 0@0].

	super drawOn: aCanvas in: drawingBounds! !

!CSkinButton methodsFor: 'drawing' stamp: 'bf 11/24/2004 17:00'!
drawShapedOn: aCanvas in: drawingBounds

	(colorize or: [fillStyle ~= #none]) ifTrue: [
		"Prefill with bg pattern"
		(colorize and: [fillAlpha < 1.0]) ifTrue: [
			| tmpForm |
			tmpForm := CSkin transparentBackgroundPattern copy: drawingBounds.
			tmpForm cutoutShape: shapeForm.
			aCanvas paintImage: tmpForm at: 0@0].
		"Fill shape"
		fillAlpha > 0.0 ifTrue: [
			aCanvas stencil: shapeForm
				at: 0@0
				sourceRect: shapeForm boundingBox
				color: fill asColor]].

	"mask our graphic"
	graphicForm ifNotNil: [
		| tmpForm |
		"Assume fit=#rigid and anchorPoint=#topLeft"
		tmpForm := graphicForm copy: (drawingBounds translateBy: graphicOffset negated).
		tmpForm cutoutShape: shapeForm.
		tmpForm drawOn: aCanvas in: drawingBounds]
! !

!CSkinButton methodsFor: 'initialize' stamp: 'pg 11/29/2005 17:19'!
initialize
	super initialize.

	colorize := false.
! !

!CSkinButton methodsFor: 'testing' stamp: 'bf 9/16/2004 17:27'!
needsUserDraw

	^ shapeForm notNil or: [colorize and: [fill isTranslucent]]! !

!CSkinButton methodsFor: 'images' stamp: 'bf 9/16/2004 17:24'!
normalImage

	^ colorize
		ifTrue: [container ifNotNil: [container normalImage]]
		ifFalse: [nil]! !

!CSkinButton methodsFor: 'events' stamp: 'bf 5/3/2004 16:31'!
onColorizeChanged
	<on: colorizeChanged>

	fillStyle := colorize
		ifTrue: [#solid]
		ifFalse: [#none].

	"set corresponding image"
	self onButtonStateChanged! !

!CSkinButton methodsFor: 'events' stamp: 'bf 4/30/2004 11:44'!
onFillChanged
	<on: fillChanged>

	self userDraw: self needsUserDraw! !

!CSkinButton methodsFor: 'events' stamp: 'ar 6/24/2005 22:23'!
onGeometryChanged
	<on: geometryChanged>

	"Keep the graphic at the origin of my container"
	graphicOffset := topLeft negated.

	"set image, it might have changed if my container was resized"
	self onButtonStateChanged! !

!CSkinButton methodsFor: 'events' stamp: 'bf 11/22/2004 15:44'!
onShapeFormChanged
	<on: shapeFormChanged>

	self userDraw: self needsUserDraw! !

!CSkinButton methodsFor: 'images' stamp: 'bf 9/16/2004 17:24'!
overImage
	^ container ifNotNil: [container overImage]! !

!CSkinButton methodsFor: 'images' stamp: 'bf 9/16/2004 17:24'!
pressedImage
	^ container ifNotNil: [container pressedImage]! !

!CSkinButton methodsFor: 'accessing' stamp: 'ar 4/1/2007 10:38'!
selectedImageSet
	"Only one set of images"
	^self! !

!CSkinButton methodsFor: 'initialize' stamp: 'pg 11/29/2005 17:20'!
setupCostume
	super setupCostume.

	graphicFit := #rigid.
	graphicAnchorPoint := #topLeft.
	
"	self onColorizeChanged.
	self onFillChanged.
"! !

!CSkinButton methodsFor: 'accessing' stamp: 'bf 4/30/2004 14:53'!
shapeForm
	"Answer the shapeForm of the receiver"
	<bewareOf: #shapeFormChanged>
	^self propertyValueAt: #shapeForm! !

!CSkinButton methodsFor: 'accessing' stamp: 'bf 4/30/2004 14:53'!
shapeForm: aValue
	"Modify the receiver's shapeForm"
	^self propertyValueAt: #shapeForm put: aValue with: #shapeFormChanged! !

!CSkinButton methodsFor: 'accessing' stamp: 'ar 4/1/2007 10:38'!
standardImageSet
	"Only one set of images"
	^self! !

!Form methodsFor: '*Tweak-Skins' stamp: 'bf 9/7/2004 15:05'!
cutoutShape: bwForm
	"use bwForm as a mask to clear all pixels except where bwForm has 1's"
	((BitBlt current destForm: self sourceForm: bwForm 
		fillColor: nil
		combinationRule: Form erase1bitShape
		destOrigin: bwForm offset 
		sourceOrigin: 0@0
		extent: self extent clipRect: self boundingBox)
		colorMap: (Bitmap with: 16rFFFFFFFF with: 0))
		copyBits.
! !

!CPlayer methodsFor: '*tweak-skins' stamp: 'bf 11/29/2004 16:22'!
shapeForm: aForm
	"SkinButton compatibility"! !

CPlayer subclass: #CSkin
	instanceVariableNames: ''
	classVariableNames: 'TransparentBackgroundPattern'
	poolDictionaries: ''
	category: 'Tweak-Skins'!

!CSkin commentStamp: 'bf 9/2/2004 11:13' prior: 0!
I'm a costume whose appearance is entirely defined by a few bitmaps:

	normalImage:	a Form with all buttons in the normal state
	overImage: 		a Form with all buttons in the mouseOver state
	pressedImage:	a Form with all buttons in the pressed state

My elements are SkinButtons, which just display a portion of these images to reflect their current state.
!

CSkin defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="normalImage" toSet="normalImage:" changeEvent="normalImageChanged">normalImage</field>
	<field toGet="overImage" toSet="overImage:" changeEvent="overImageChanged">overImage</field>
	<field toGet="pressedImage" toSet="pressedImage:" changeEvent="pressedImageChanged">pressedImage</field>
</fields>'!

!CSkin class methodsFor: 'patterns' stamp: 'bf 9/2/2004 16:20'!
patternSize: patternSize color: color1 and: color2
	| pattern h |
	h := patternSize // 2.
	(pattern := ColorForm extent: patternSize@patternSize depth: 2)
		fillColor: Color gray;							"pixelValue 3"
		fill: (0@0 extent: h@h) fillColor: Color black;		"pixelValue 1"
		fill: (h@h extent: h@h) fillColor: Color black;		"pixelValue 1"
		colors: {Color transparent. color1. Color red. color2}.
	^pattern
! !

!CSkin class methodsFor: 'patterns' stamp: 'bf 9/2/2004 16:20'!
transparentBackgroundPattern
	^ TransparentBackgroundPattern ifNil: [
		| smallPattern |
		smallPattern := InfiniteForm with:
			(self patternSize: 8 color: Color darkGray and: Color veryLightGray).
		TransparentBackgroundPattern := InfiniteForm with:
			(smallPattern copy: (0@0 extent: 64@64))].
! !

!CSkin methodsFor: 'testing' stamp: 'bf 9/2/2004 11:05'!
containsPoint: aPoint

	graphicForm ifNil: [^super containsPoint: aPoint].
	^ (graphicForm pixelValueAt: aPoint) ~= 0! !

!CSkin methodsFor: 'initialize' stamp: 'bf 9/7/2004 11:41'!
initialize
	super initialize.

	self initializeButtons.

! !

!CSkin methodsFor: 'initialize' stamp: 'bf 9/7/2004 11:40'!
initializeButtons

	^self subclassResponsibility
! !

!CSkin methodsFor: 'initialize' stamp: 'bf 4/28/2004 16:20'!
makeDefaultButton
	
	^ CSkinButton new! !

!CSkin methodsFor: 'initialize' stamp: 'bf 9/7/2004 16:28'!
makeNormalImage
	"Required"
	^ self subclassResponsibility! !

!CSkin methodsFor: 'initialize' stamp: 'bf 9/7/2004 16:28'!
makeOverImage
	"Optional"
	^ nil! !

!CSkin methodsFor: 'initialize' stamp: 'bf 9/7/2004 16:28'!
makePressedImage
	"Optional"
	^ nil! !

!CSkin methodsFor: 'accessing' stamp: 'bf 5/6/2004 15:35'!
normalImage
	"Answer the normalImage of the receiver"
	<bewareOf: #normalImageChanged>
	^self propertyValueAt: #normalImage! !

!CSkin methodsFor: 'accessing' stamp: 'bf 5/6/2004 15:35'!
normalImage: aValue
	"Modify the receiver's normalImage"
	^self propertyValueAt: #normalImage put: aValue with: #normalImageChanged! !

!CSkin methodsFor: 'events' stamp: 'bf 4/27/2004 14:37'!
onGraphicChanged
	<on: graphicChanged>

	(graphicForm notNil and: [extent ~= graphicForm extent]) ifTrue: [
		extent := graphicForm extent].
! !

!CSkin methodsFor: 'accessing' stamp: 'bf 3/29/2004 12:40'!
overImage
	"Answer the overImage of the receiver"
	<bewareOf: #overImageChanged>
	^self propertyValueAt: #overImage! !

!CSkin methodsFor: 'accessing' stamp: 'bf 3/29/2004 12:40'!
overImage: aValue
	"Modify the receiver's overImage"
	^self propertyValueAt: #overImage put: aValue with: #overImageChanged! !

!CSkin methodsFor: 'accessing' stamp: 'bf 3/29/2004 12:40'!
pressedImage
	"Answer the pressedImage of the receiver"
	<bewareOf: #pressedImageChanged>
	^self propertyValueAt: #pressedImage! !

!CSkin methodsFor: 'accessing' stamp: 'bf 3/29/2004 12:40'!
pressedImage: aValue
	"Modify the receiver's pressedImage"
	^self propertyValueAt: #pressedImage put: aValue with: #pressedImageChanged! !

!CSkin methodsFor: 'initialize' stamp: 'bf 9/7/2004 11:39'!
setupButtons

	^self subclassResponsibility
! !

!CSkin methodsFor: 'initialize' stamp: 'bf 9/16/2004 16:08'!
setupCostume
	super setupCostume.

	self setupImages.
	self setupButtons.
	self setupCursors.

	graphicForm := normalImage.
	graphicFit := #rigid.
	graphicAnchorPoint := #topLeft.
	self userDraw: true. 		"for containsPoint:"

	self onGraphicChanged. "init extent"

! !

!CSkin methodsFor: 'initialize' stamp: 'bf 9/16/2004 16:09'!
setupCursors
	"no default cursors"
	! !

!CSkin methodsFor: 'initialize' stamp: 'bf 9/7/2004 16:26'!
setupImages

	normalImage := self makeNormalImage.
	pressedImage := self makePressedImage.
	overImage := self makeOverImage.
! !
